@@ -3,7 +3,6 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-# pyre-strict
 from dataclasses import dataclass, fields
 from typing import Generator, List, Optional, Sequence, Set, Tuple, Type, Union
 
@@ -455,7 +454,6 @@ generated_code.append(
 )
 generated_code.append("# LICENSE file in the root directory of this source tree.")
 generated_code.append("")
-generated_code.append("# pyre-strict")
 generated_code.append("")
 generated_code.append("# This file was generated by libcst.codegen.gen_matcher_classes")
 generated_code.append("from abc import ABC")
