@@ -77,6 +77,16 @@ class RemoveUnusedImportsCommandTest(CodemodTest):
 
         self.assertCodemod(before, after)
 
+    def test_enclosed_attributes(self) -> None:
+        before = """
+            from a.b import c
+            import x
+
+            def foo() -> None:
+                x.y(c.d()).z()
+        """
+        self.assertCodemod(before, before)
+
     def test_access_in_assignment(self) -> None:
         before = """
             from a import b
@@ -90,3 +100,33 @@ class RemoveUnusedImportsCommandTest(CodemodTest):
             a(b, 'look at these ugly quotes')
         """
         self.assertCodemod(before, before)
+
+    def test_suppression_on_first_line_of_multiline_import_refers_to_whole_block(
+        self,
+    ) -> None:
+        before = """
+            from a import (  # lint-ignore: unused-import
+                b,
+                c,
+            )
+        """
+        self.assertCodemod(before, before)
+
+    def test_suppression(self) -> None:
+        before = """
+            # noqa
+            import a, b
+            import c
+            from x import (
+                y,
+                z,  # noqa
+            )
+        """
+        after = """
+            # noqa
+            import a, b
+            from x import (
+                z,  # noqa
+            )
+        """
+        self.assertCodemod(before, after)
