@@ -21,6 +21,7 @@ from libcst.metadata.scope_provider import (
     QualifiedNameSource,
     Scope,
     ScopeProvider,
+    _gen_dotted_names,
 )
 from libcst.testing.utils import UnitTest, data_provider
 
@@ -140,6 +141,16 @@ class ScopeProviderTest(UnitTest):
             """
         )
         scope_of_module = scopes[m]
+
+        import_0 = cst.ensure_type(
+            cst.ensure_type(m.body[0], cst.SimpleStatementLine).body[0], cst.Import
+        )
+        self.assertEqual(scopes[import_0], scope_of_module)
+        import_aliases = import_0.names
+        if not isinstance(import_aliases, cst.ImportStar):
+            for alias in import_aliases:
+                self.assertEqual(scopes[alias], scope_of_module)
+
         for idx, in_scopes in enumerate(
             [["foo", "foo.bar"], ["fizzbuzz"], ["a", "a.b", "a.b.c"], ["g"],]
         ):
@@ -193,6 +204,39 @@ class ScopeProviderTest(UnitTest):
         self.assertEqual(list(scope_of_module["x.y"])[0].references, set())
         self.assertEqual(scope_of_module.accesses["x.y"], set())
 
+    def test_dotted_import_with_call_access(self) -> None:
+        m, scopes = get_scope_metadata_provider(
+            """
+            import os.path
+            os.path.join("A", "B").lower()
+            """
+        )
+        scope_of_module = scopes[m]
+        first_statement = ensure_type(m.body[1], cst.SimpleStatementLine)
+        attr = ensure_type(
+            ensure_type(
+                ensure_type(
+                    ensure_type(
+                        ensure_type(first_statement.body[0], cst.Expr).value, cst.Call
+                    ).func,
+                    cst.Attribute,
+                ).value,
+                cst.Call,
+            ).func,
+            cst.Attribute,
+        ).value
+        self.assertTrue("os.path" in scope_of_module)
+        self.assertTrue("os" in scope_of_module)
+
+        os_path_join_assignment = cast(Assignment, list(scope_of_module["os.path"])[0])
+        os_path_join_assignment_references = list(os_path_join_assignment.references)
+        self.assertNotEqual(len(os_path_join_assignment_references), 0)
+        os_path_join_access = os_path_join_assignment_references[0]
+        self.assertEqual(scope_of_module.accesses["os"], set())
+        self.assertEqual(scope_of_module.accesses["os.path"], {os_path_join_access})
+        self.assertEqual(scope_of_module.accesses["os.path.join"], set())
+        self.assertEqual(os_path_join_access.node, attr)
+
     def test_import_from(self) -> None:
         m, scopes = get_scope_metadata_provider(
             """
@@ -202,6 +246,16 @@ class ScopeProviderTest(UnitTest):
             """
         )
         scope_of_module = scopes[m]
+
+        import_from = cst.ensure_type(
+            cst.ensure_type(m.body[0], cst.SimpleStatementLine).body[0], cst.ImportFrom
+        )
+        self.assertEqual(scopes[import_from], scope_of_module)
+        import_aliases = import_from.names
+        if not isinstance(import_aliases, cst.ImportStar):
+            for alias in import_aliases:
+                self.assertEqual(scopes[alias], scope_of_module)
+
         for idx, in_scope in [(0, "a"), (0, "b_renamed"), (1, "c"), (2, "d")]:
             self.assertEqual(
                 len(scope_of_module[in_scope]), 1, f"{in_scope} should be in scope."
@@ -235,8 +289,9 @@ class ScopeProviderTest(UnitTest):
             """
         )
         scope_of_module = scopes[m]
-        func_body = ensure_type(m.body[1], cst.FunctionDef).body
-        func_body_statement = func_body.body[0]
+        func_def = ensure_type(m.body[1], cst.FunctionDef)
+        self.assertEqual(scopes[func_def], scopes[func_def.name])
+        func_body_statement = func_def.body.body[0]
         scope_of_func = scopes[func_body_statement]
         self.assertIsInstance(scope_of_func, FunctionScope)
         self.assertTrue("global_var" in scope_of_module)
@@ -266,6 +321,7 @@ class ScopeProviderTest(UnitTest):
         cls_assignment = cast(Assignment, cls_assignments[0])
         cls_def = ensure_type(m.body[1], cst.ClassDef)
         self.assertEqual(cls_assignment.node, cls_def)
+        self.assertEqual(scopes[cls_def], scopes[cls_def.name])
         cls_body = cls_def.body
         cls_body_statement = cls_body.body[0]
         scope_of_class = scopes[cls_body_statement]
@@ -1131,3 +1187,34 @@ class ScopeProviderTest(UnitTest):
         scope.get_qualified_names_for("doesnt_exist")
         self.assertEqual(len(scope._assignments), assignments_len_before)
         self.assertEqual(len(scope._accesses), accesses_len_before)
+
+    def test_gen_dotted_names(self) -> None:
+        names = {name for name, node in _gen_dotted_names(cst.Name(value="a"))}
+        self.assertEqual(names, {"a"})
+
+        names = {
+            name
+            for name, node in _gen_dotted_names(
+                cst.Attribute(value=cst.Name(value="a"), attr=cst.Name(value="b"))
+            )
+        }
+        self.assertEqual(names, {"a.b", "a"})
+
+        names = {
+            name
+            for name, node in _gen_dotted_names(
+                cst.Attribute(
+                    value=cst.Call(
+                        func=cst.Attribute(
+                            value=cst.Attribute(
+                                value=cst.Name(value="a"), attr=cst.Name(value="b")
+                            ),
+                            attr=cst.Name(value="c"),
+                        ),
+                        args=[],
+                    ),
+                    attr=cst.Name(value="d"),
+                )
+            )
+        }
+        self.assertEqual(names, {"a.b.c", "a.b", "a"})
